@@ -10,9 +10,9 @@ GUILE_LOAD_PATH=$topdir/examples:$GUILE_LOAD_PATH
 GUILE_LOAD_PATH=$topdir/test-suite:$GUILE_LOAD_PATH
 export GUILE_LOAD_PATH
 
-if [ "X$GUILE_LOAD_COMPILED_PATH" = "X" ]; then
- GUILE_LOAD_COMPILED_PATH=$topdir/examples
-else
- GUILE_LOAD_COMPILED_PATH=$topdir/examples:$GUILE_LOAD_COMPILED_PATH
-fi;
-export GUILE_LOAD_COMPILED_PATH
+#if [ "X$GUILE_LOAD_COMPILED_PATH" = "X" ]; then
+# GUILE_LOAD_COMPILED_PATH=$topdir/examples
+#else
+# GUILE_LOAD_COMPILED_PATH=$topdir/examples:$GUILE_LOAD_COMPILED_PATH
+#fi;
+#export GUILE_LOAD_COMPILED_PATH
