@@ -1,20 +1,19 @@
 (define redo #t)
 ;; Tmach.scm - javascript
 ;;
-;; Copyright (C) 2015,2016 Matthew R. Wette
+;; Copyright (C) 2015,2016,2018 Matthew R. Wette
 ;; 
 ;; Copying and distribution of this file, with or without modification,
 ;; are permitted in any medium without royalty provided the copyright
 ;; notice and this notice are preserved.  This file is offered as-is,
 ;; without any warranty.
 
-(add-to-load-path (string-append (getcwd) "/../../../../module/"))
-(add-to-load-path (string-append (getcwd) "/../../../../examples/"))
-
 (use-modules (nyacc lang javascript mach))
+
 (when redo
-  (gen-js-files) (system "touch parser.scm")
-  (gen-se-files) (system "touch separser.scm"))
+  (gen-javascript-files)
+  ;;(system "touch parser.scm")
+  )
 
 (use-modules (nyacc lang javascript parser))
 (use-modules (nyacc lang javascript pprint))
@@ -25,28 +24,24 @@
 (use-modules (ice-9 pretty-print))
 
 (when redo
-  (with-output-to-file ",lang.txt"
+  (with-output-to-file ",file.txt"
     (lambda ()
-      (pp-lalr-notice js-spec)
-      (pp-lalr-grammar js-spec)
-      (pp-lalr-machine js-mach))))
+      (pp-lalr-notice javascript-spec)
+      (pp-lalr-grammar javascript-spec)
+      (pp-lalr-machine javascript-mach))))
 
-(when #f
-  (with-output-to-file ",selang.txt"
+(when redo
+  (with-output-to-file ",stmt.txt"
     (lambda ()
-      (let* ((spec (restart-spec js-spec 'SourceElement))
-	     (mach (compact-machine
-		    (hashify-machine
-		     (make-lalr-machine spec)))))
-      (pp-lalr-notice spec)
-      (pp-lalr-grammar spec)
-      (pp-lalr-machine mach)))))
+	;;(pp-lalr-notice javascript-ia-spec)
+	(pp-lalr-grammar javascript-ia-spec)
+	(pp-lalr-machine javascript-ia-mach))))
 
 (when #f
   (with-output-to-file "gram.y.new"
-    (lambda () (lalr->bison js-spec))))
+    (lambda () (lalr->bison javascript-spec))))
 
-(when #t
+(when #f
   (let* ((xargs (cdr (program-arguments)))
 	 (file (if (pair? xargs) (car xargs) ",ex1.js"))
 	 (res (with-input-from-file file dev-parse-js)))
