@@ -39,7 +39,7 @@
     (make-ident-reader idf idr)))
 
 ;; @deffn {Procecure} read-js-string ch => ($string "abc") | #f
-(define (read-js-string delim)
+(define-public (read-js-string delim)
   (if (not (or (eq? delim #\') (eq? delim #\"))) #f
       (let iter ((cl '()) (ch (read-char)))
 	(cond ((eq? ch #\\)
@@ -59,55 +59,59 @@
 	      ((eq? ch delim) (cons '$string (list->string (reverse cl))))
 	      (else (iter (cons ch cl) (read-char)))))))
 
-(define gen-js-lexer
-  (let* ((match-table mtab)
-	 (space-cs (string->char-set " \t\r"))
+;; Add #! ... !# to comment format so that we can use shebang scripts.
+(define read-js-comm
+  (make-comm-reader '(("/*" . "*/") ("//" . "\n")
+		      ("#!" . "!#") ("#lang" . "\n"))))
+
+;; maybe turn double newline into ';'
+(define-public (make-js-lexer-generator match-table)
+  (let* ((space-cs (string->char-set " \t\r"))
 	 ;;
-	 (strtab (filter-mt string? match-table)) ; strings in grammar
-	 (kwstab (filter-mt like-c-ident? strtab)) ; keyword strings =>
-	 (keytab (map-mt string->symbol kwstab)) ; keywords in grammar
-	 (chrseq (remove-mt like-c-ident? strtab))  ; character sequences
-	 (symtab (filter-mt symbol? match-table)) ; symbols in grammar
-	 (chrtab (filter-mt char? match-table))	; characters in grammar
+	 (strtab (filter-mt string? match-table))  ; strings in gram
+	 (kwstab (filter-mt like-c-ident? strtab)) ; keywd strings =>
+	 (keytab (map-mt string->symbol kwstab))   ; keywds in gram
+	 (chrseq (remove-mt like-c-ident? strtab)) ; character seq's
+	 (symtab (filter-mt symbol? match-table))  ; sym's in gram
+	 (chrtab (filter-mt char? match-table))	   ; char's in gram
 	 ;;
 	 (read-chseq (make-chseq-reader chrseq))
 	 (semicolon (assoc-ref chrseq ";"))
-	 (assc-$ (lambda (pair) (cons (assq-ref symtab (car pair)) (cdr pair))))
-	 )
+	 (assc-$ (lambda (pair) (cons (assq-ref symtab (car pair)) (cdr pair)))))
     (lambda ()
       (let ((bol #t))
 	(lambda ()
-          (define (cont rez) (fluid-set! *insert-semi* #t) rez)
-          (cont
-	  (let iter ((ch (read-char)))
-	    (cond
-	     ((eof-object? ch) (assc-$ (cons '$end ch)))
-	     ((char-set-contains? space-cs ch) (iter (read-char)))
-	     ((eqv? ch #\newline)
-	      (if (fluid-ref *insert-semi*)
-		  (cons semicolon ";")
-		  (iter (read-char))))
-	     ((and (eqv? ch #\newline) (set! bol #t) #f))
-	     ((read-js-string ch) => assc-$)
-	     ((read-c-comm ch bol) (iter (read-char)))
-	     ((read-js-ident ch) =>
-	      (lambda (s)
-		(or (and=> (assq-ref keytab (string->symbol s))
-			   (lambda (tval) (cons tval s)))
-		    (assc-$ (cons '$ident s)))))
-	     ((read-c-num ch) => assc-$)
-	     ((read-chseq ch) => identity)
-	     ((assq-ref chrtab ch) => (lambda (t) (cons t (string ch))))
-	     (else (cons ch (string ch))))))))))) ; should be error
+          (define (process lexeme) (fluid-set! *insert-semi* #t) lexeme)
+          (process
+	   (let iter ((ch (read-char)))
+	     (cond
+	      ((eof-object? ch) (assc-$ (cons '$end ch)))
+	      ((char-set-contains? space-cs ch) (iter (read-char)))
+	      ((eqv? ch #\newline)
+	       (set! bol #t)
+	       (if (fluid-ref *insert-semi*)
+		   (cons semicolon "\n")
+		   (iter (read-char))))
+	      ((read-js-comm ch bol) (iter (read-char)))
+	      ((and (set! bol #f) #f))
+	      ((read-js-string ch) => assc-$)
+	      ((read-js-ident ch) =>
+	       (lambda (s)
+		 (or (and=> (assq-ref keytab (string->symbol s))
+			    (lambda (tval) (cons tval s)))
+		     (assc-$ (cons '$ident s)))))
+	      ((read-c-num ch) => assc-$)
+	      ((read-chseq ch) => identity)
+	      ((assq-ref chrtab ch) => (lambda (t) (cons t (string ch))))
+	      (else (cons ch (string ch))))))))))) ; should be error
 
-#|
-    (InputElementDiv
-     (WhiteSpace) (LineTerminator) (Comment) (Token) (DivPunctuator))
-    (InputElementRegExp
-     (WhiteSpace) (LineTerminator) (Comment) (Token) (RegularExpressionLiteral))
-    (WhiteSpace ("\t") ("\vt") ("\ff") (" ") ("&nbsp;") ("&usp;"))
-    (LineTerminator ("\n") ("\r") (LS) (PS))
-    (MultiLineComment ('multiline-comment))
-|#
+
+;; (InputElementDiv
+;;  (WhiteSpace) (LineTerminator) (Comment) (Token) (DivPunctuator))
+;; (InputElementRegExp
+;;  (WhiteSpace) (LineTerminator) (Comment) (Token) (RegularExpressionLiteral)
+;;  (WhiteSpace ("\t") ("\vt") ("\ff") (" ") ("&nbsp;") ("&usp;"))
+;; (LineTerminator ("\n") ("\r") (LS) (PS))
+;; (MultiLineComment ('multiline-comment))
 
 ;;; --- last line ---
