@@ -16,24 +16,23 @@
 ;; along with this library; if not, see <http://www.gnu.org/licenses/>.
 
 (define-module (nyacc lang javascript mach)
-  #:export (js-spec
-	    js-mach
+  #:export (javascript-spec
+	    javascript-mach
+	    javascript-ia-spec
+	    javascript-ia-mach
 	    dev-parse-js
-	    gen-js-files gen-se-files)
+	    gen-javascript-files)
   #:use-module (nyacc lang util)
   #:use-module (nyacc lalr)
   #:use-module (nyacc parse)
   #:use-module (nyacc lex)
   #:use-module (nyacc util)
-  )
-(cond-expand
- (guile-2
-  (use-modules ((srfi srfi-43) #:select (vector-map))))
- (else
-  (use-modules (nyacc compat18))))
-  
+  #:use-module ((srfi srfi-43) #:select (vector-map)))
 
-;; This parses EcmaScript v3 1999.  Some v5 2011 items are added as comments.
+;; This was started with EcmaScript v3 sped.
+;; Some v5 2011 items are added as comments.
+;; Also added "let".
+;; Added: "var" not allowed in blocks, only "let"
 
 ;; The 'NoIn' variants are needed to avoid confusing the in operator 
 ;; in a relational expression with the in operator in a for statement.
@@ -42,9 +41,16 @@
 
 ;; NSI = "no semi-colon insertion"
 
-(define js-spec
+;; @subheading Deviations from Ecmascript
+;; @itemize
+;; @item Supports @code{let} in block context
+;; @item @code{var} is not allowed in block context
+;; @end itemize
+
+(define javascript-spec
   (lalr-spec
-   (notice (string-append "Copyright 2015-2017 Matthew R. Wette" lang-crn-lic))
+   (notice (string-append "Copyright 2015-2018 Matthew R. Wette"
+			  license-lgpl3+))
    (reserve "abstract" "boolean" "byte" "char" "class" "const" "debugger"
 	    "double" "enum" "export" "extends" "final" "float" "goto"
 	    "implements" "import" "int" "interface" "long" "native" 
@@ -374,16 +380,14 @@
       "," AssignmentExpression
       ($$ (if (and (pair? (car $1)) (eqv? 'expr-list (caar $1)))
 	      (tl-append $1 $3)
-	      (make-tl 'expr-list $1 $3))))
-     )
+	      (make-tl 'expr-list $1 $3)))))
     (ExpressionNoIn
      (AssignmentExpressionNoIn)
      (ExpressionNoIn
       "," AssignmentExpressionNoIn
       ($$ (if (and (pair? (car $1)) (eqv? 'expr-list (caar $1)))
 	      (tl-append $1 $3)
-	      (make-tl 'expr-list $1 $3))))
-     )
+	      (make-tl 'expr-list $1 $3)))))
 	    
     ;; A.4
     (Statement
@@ -404,30 +408,42 @@
      ;;(DebuggerStatement) v5.1
      )
 
+    ;; Allow "var" statements in block but only at beginning and only
+    ;; at function block entry, by static semantics so we can generate
+    ;; error messages.
     (Block
-     ("{" StatementList "}" ($prec 'stmt) ($$ `(Block . ,(cdr (tl->list $2)))))
+     ;;("{" StatementList "}" ($prec 'stmt) ($$ `(Block . ,(cdr (tl->list $2)))))
+     ("{" LetStatementList StatementList "}" ($prec 'stmt)
+      ($$ `(Block . ,(append (sx-tail (tl->list $2))
+			     (sx-tail (tl->list $3))))))
+     ("{" StatementList "}" ($prec 'stmt)
+      ($$ `(Block . ,(sx-tail (tl->list $2)))))
      ("{" "}" ($prec 'stmt) ($$ '(Block)))
      )
 
+    (LetStatementList
+     (LetStatement ($$ (make-tl 'LetStatementList $1)))
+     (LetStatementList LetStatement ($$ (tl-append $1 $2))))
+
+    (LetStatement
+     ("let" DeclarationList ";"
+      ($$ `(LetStatement ,(tl->list $2)))))
+
     (StatementList
      (Statement ($$ (make-tl 'StatementList $1)))
-     (StatementList Statement ($$ (tl-append $1 $2)))
-     )
+     (StatementList Statement ($$ (tl-append $1 $2))))
 
     (VariableStatement
-     ("var" VariableDeclarationList ";"
-      ($$ `(VariableStatement ,(tl->list $2))))
-     )
+     ("var" DeclarationList ";"
+       ($$ `(VariableStatement ,(tl->list $2)))))
 
-    (VariableDeclarationList
-     (VariableDeclaration ($$ (make-tl 'VariableDeclarationList $1)))
-     (VariableDeclarationList "," VariableDeclaration ($$ (tl-append $1 $3)))
-     )
-    (VariableDeclarationListNoIn
-     (VariableDeclarationNoIn ($$ (make-tl 'VariableDeclarationList $1)))
-     (VariableDeclarationListNoIn "," VariableDeclarationNoIn
-				  ($$ (tl-append $1 $3)))
-     )
+    (DeclarationList
+     (VariableDeclaration ($$ (make-tl 'DeclarationList $1)))
+     (DeclarationList "," VariableDeclaration ($$ (tl-append $1 $3))))
+    (DeclarationListNoIn
+     (VariableDeclarationNoIn ($$ (make-tl 'DeclarationList $1)))
+     (DeclarationListNoIn "," VariableDeclarationNoIn
+				  ($$ (tl-append $1 $3))))
 
     (VariableDeclaration
      (Identifier Initializer ($$ `(VariableDeclaration ,$1 ,$2)))
@@ -467,7 +483,7 @@
       ($$ `(while ,$3 ,$5)))
      ("for" "(" OptExprStmtNoIn OptExprStmt OptExprClose Statement
       ($$ `(for $3 $4 $5 $6)))
-     ("for" "(" "var" VariableDeclarationListNoIn ";" OptExprStmt
+     ("for" "(" "var" DeclarationListNoIn ";" OptExprStmt
       OptExprClose Statement
       ($$ `(for $4 $6 $7 $8)))		; ???
      ("for" "(" LeftHandSideExpression "in" Expression ")" Statement
@@ -601,95 +617,79 @@
 
     (FormalParameterList
      (Identifier ($$ (make-tl 'FormalParameterList $1)))
-     (FormalParameterList "," Identifier ($$ (tl-append $1 $3)))
-     )
+     (FormalParameterList "," Identifier ($$ (tl-append $1 $3))))
 
+    ;; I have duplicated SourceElement as FunctionElement and ProgramElement
+    ;; in order to allow this grammar to be reused for interactive parser.
+    ;; The ia-parser will use ProgramElement as the start symbol.
+    
     (FunctionBody
-     (SourceElements ($$ (tl->list $1)))
-     )
+     (FunctionElements ($$ (tl->list $1))))
 
-    (Program
-     (SourceElements ($$ (list 'Program (tl->list $1))))
-     )
+    (FunctionElements
+     (FunctionElement ($$ (make-tl 'FunctionElements $1)))
+     (FunctionElements FunctionElement ($$ (tl-append $1 $2))))
 
-    (SourceElements
-     (SourceElement ($$ (make-tl 'SourceElements $1)))
-     (SourceElements SourceElement ($$ (tl-append $1 $2)))
-     )
-
-    (SourceElement
+    (FunctionElement
      (Statement)
-     (FunctionDeclaration)
-     )
+     (FunctionDeclaration))
+    
+    (Program
+     (ProgramElements ($$ `(Program ,(tl->list $1)))))
+
+    (ProgramElements
+     (ProgramElement ($$ (make-tl 'ProgramElements $1)))
+     (ProgramElements ProgramElement ($$ (tl-append $1 $2))))
+
+    (ProgramElement
+     (Statement)
+     (FunctionDeclaration))
     
     )))
 
-(define js-mach
+;; === parsers ==========================
+
+(define javascript-mach
   (hashify-machine
    (compact-machine
-    (make-lalr-machine js-spec))))
-
-(define len-v (assq-ref js-mach 'len-v))
-(define pat-v (assq-ref js-mach 'pat-v))
-(define rto-v (assq-ref js-mach 'rto-v))
-(define mtab (assq-ref js-mach 'mtab))
-(define sya-v (vector-map (lambda (ix actn) (wrap-action actn))
-			  (assq-ref js-mach 'act-v)))
-(define act-v (vector-map (lambda (ix f) (eval f (current-module))) sya-v))
+    (make-lalr-machine javascript-spec))))
 
 (include-from-path "nyacc/lang/javascript/body.scm")
 
-(define raw-parser (make-lalr-parser js-mach))
+(define gen-js-lexer (make-js-lexer-generator (assq-ref javascript-mach 'mtab)))
+
+(define raw-parser (make-lalr-parser javascript-mach))
 
 (define* (dev-parse-js #:key debug)
-  (catch
-   'nyacc-error
+  (catch 'nyacc-error
    (lambda ()
-     (with-fluid*
-	 *insert-semi* #t
-	 (lambda () (raw-parser (gen-js-lexer) #:debug debug))))
+     (with-fluid* *insert-semi* #t
+       (lambda () (raw-parser (gen-js-lexer) #:debug debug))))
    (lambda (key fmt . args)
      (report-error fmt args)
      #f)))
 
-;; ======= gen files
+(define javascript-ia-spec (restart-spec javascript-spec 'ProgramElement))
 
-(define (gen-js-files . rest)
-  (define (lang-dir path)
-    (if (pair? rest) (string-append (car rest) "/" path) path))
-  (define (xtra-dir path)
-    (lang-dir (string-append "mach.d/" path)))
+(define javascript-ia-mach
+  (hashify-machine
+   (compact-machine
+    (make-lalr-machine javascript-ia-spec))))
 
-  (write-lalr-actions js-mach (xtra-dir "jsact.scm.new"))
-  (write-lalr-tables js-mach (xtra-dir "jstab.scm.new"))
-  (let ((a (move-if-changed (xtra-dir "jsact.scm.new")
-			    (xtra-dir "jsact.scm")))
-	(b (move-if-changed (xtra-dir "jstab.scm.new")
-			    (xtra-dir "jstab.scm"))))
-    (when (or a b) 
-      (system (string-append "touch " (lang-dir "parser.scm"))))
-    (or a b)))
-
-(define (gen-se-files . rest)
-  (define (lang-dir path)
-    (if (pair? rest) (string-append (car rest) "/" path) path))
-  (define (xtra-dir path)
-    (lang-dir (string-append "mach.d/" path)))
-
-  (let* ((se-spec (restart-spec js-spec 'SourceElement))
-	 (se-mach (make-lalr-machine se-spec))
-	 (se-mach (compact-machine se-mach))
-	 (se-mach (hashify-machine se-mach)))
-    (write-lalr-actions se-mach (xtra-dir "seact.scm.new"))
-    (write-lalr-tables se-mach (xtra-dir "setab.scm.new")))
-  
-  (let ((a (move-if-changed (xtra-dir "seact.scm.new")
-			    (xtra-dir "seact.scm")))
-	(b (move-if-changed (xtra-dir "setab.scm.new")
-			    (xtra-dir "setab.scm"))))
-    #;(when (or a b)
-    (system (string-append "touch " (lang-dir "separser.scm"))))
-    (or a b)))
+;; === automaton file generators =========
 
+(define* (gen-javascript-files #:optional (path "."))
+  (define (mdir file) (mach-dir path file))
+  (write-lalr-actions javascript-mach (mdir "jsact.scm.new") #:prefix "js-")
+  (write-lalr-tables javascript-mach (mdir "jstab.scm.new") #:prefix "js-")
+  (write-lalr-actions javascript-ia-mach
+		      (mdir "ia-jsact.scm.new") #:prefix "ia-js-")
+  (write-lalr-tables javascript-ia-mach
+		     (mdir "ia-jstab.scm.new") #:prefix "ia-js-")
+  (let ((a (move-if-changed (mdir "jsact.scm.new") (mdir "jsact.scm")))
+	(b (move-if-changed (mdir "jstab.scm.new") (mdir "jstab.scm")))
+	(c (move-if-changed (mdir "ia-jsact.scm.new") (mdir "ia-jsact.scm")))
+	(d (move-if-changed (mdir "ia-jstab.scm.new") (mdir "ia-jstab.scm"))))
+    (or a b c d)))
 
 ;;; --- last line ---
