@@ -33,7 +33,6 @@
 (use-modules (nyacc lang util))
 (use-modules ((srfi srfi-1) #:select (fold-right)))
 (use-modules ((srfi srfi-9) #:select (define-record-type)))
-(use-modules (ice-9 regex))
 (use-modules (ice-9 pretty-print))	; for debugging
 
 ;; C parser info (?)
@@ -207,20 +206,6 @@
   (for-each add-typename (find-new-typenames decl))
   decl)
 
-;; NOT USED NOW
-(define gen-attributes
-  (let ((rx (make-regexp "^__(.*)__$")))
-    (lambda (spec)
-      ;; spec is '(@ ("packed" "packed") ...)
-      (sx-match spec
-	((attributes ,name)
-	 (cond
-	  ((regexp-exec rx name) =>
-	   (lambda (m) (cons (match:substring m 1) "")))
-	  (else
-	   (cons name ""))))
-	))))
-
 ;; used in c99-spec actions for attribute-specifiers
 (define (attr-expr-list->string attr-expr-list)
   (string-append "(" (string-join (cdr attr-expr-list) ",") ")"))
@@ -267,8 +252,18 @@
 (define (def-xdef? name mode)
   (not (eqv? mode 'file)))
 
-;; @deffn {Procedure} gen-c-lexer [#:mode mode] [#:xdef? proc] => procedure
-;; Generate a context-sensitive lexer for the C99 language.  The generated
+  
+;; @deffn {Procedure} make-c99-lexer-generator match-table raw-parser => proc
+;; This generates a procedure which has the signature
+;; @example
+;; proc [#:mode mode] [#:xdef? proc] => procedure
+;; @end example
+;; to be passed to the c99 parsers.
+;; The proc will generate a context-sensitive lexer for the C99 language.
+;; The arg @var{match-table} is an element of a specification returned
+;; by @code{make-lalr-spec} or machine generated by @code{make-lalr-machine}.
+;; The argument @var{raw-parse} must be ...
+;; The generated
 ;; lexical analyzer reads and passes comments and optionally CPP statements
 ;; to the parser.  The keyword argument @var{mode} will determine if CPP
 ;; statements are passed (@code{'file} mode) or parsed and executed
@@ -281,7 +276,7 @@
 ;; (define (def-xdef? mode name) (eqv? mode 'code))
 ;; @end example
 ;; @end deffn
-(define gen-c-lexer
+(define (make-c99-lexer-generator match-table raw-parser)
   ;; This gets ugly in order to handle cpp.  The CPP will tokenize, expand,
   ;; then convert back to a string.
   ;;
@@ -296,8 +291,20 @@
   ;; inserted via a string-port on the port stack.  When that port is fully
   ;; read (i.e., the reader sees eof-object) then @var{suppress} is changed
   ;; to @code{#t}.
-  (let* ((match-table c99-mtab)
-	 (ident-like? (make-ident-like-p read-c-ident))
+
+  (define (getdefs stmts)		; extract defines
+    (fold-right
+     (lambda (stmt seed)
+       ;;(sx-match stmt
+       ;;  ((cpp-stmt (define . ,rest)) (cons (sx-ref stmt 1) seed))
+       ;;  (else seed)))
+       (if (and (eqv? 'cpp-stmt (sx-tag stmt))
+		(eqv? 'define (sx-tag (sx-ref stmt 1))))
+	   (cons (sx-ref stmt 1) seed)
+	   seed))
+     '() stmts))
+  
+  (let* ((ident-like? (make-ident-like-p read-c-ident))
 	 ;;
 	 (strtab (filter-mt string? match-table)) ; strings in grammar
 	 (kwstab (filter-mt ident-like? strtab))  ; keyword strings =>
@@ -308,20 +315,21 @@
 	 ;;
 	 (read-chseq (make-chseq-reader chrseq))
 	 (assc-$ (lambda (pair) (cons (assq-ref symtab (car pair)) (cdr pair))))
+
 	 ;;
 	 (t-ident (assq-ref symtab '$ident))
-	 (t-typename (assq-ref symtab 'typename))
-	 (xp1 (lambda (stmts)
-		(fold-right
-		 (lambda (stmt seed)
-		   (if (and (eqv? 'cpp-stmt (sx-tag stmt))
-			    (eqv? 'define (sx-tag (sx-ref stmt 1))))
-		       (cons (sx-ref stmt 1) seed)
-		       seed))
-		 '() stmts))))
+	 (t-typename (assq-ref symtab 'typename)))
+
     ;; mode: 'code|'file|'decl
     ;; xdef?: (proc name mode) => #t|#f  : do we expand #define?
-    (lambda* (#:key (mode 'code) xdef? show-incs)
+    ;;(lambda* (#:key (mode 'code) xdef? show-incs)
+    (define* (lexer #:key (mode 'code) xdef? show-incs)
+
+      (define (run-parse)
+	(let ((info (fluid-ref *info*)))
+	  (raw-parser (lexer #:mode 'decl #:show-incs (cpi-shinc info))
+		      #:debug (cpi-debug info))))
+      
       (let ((bol #t)		 ; begin-of-line condition
 	    (suppress #f)	 ; parsing cpp expanded text (kludge?)
 	    (ppxs (list 'keep))	 ; CPP execution state stack
@@ -329,8 +337,7 @@
 	    ;;(brlev 0)			; brace level
 	    (x-def? (cond ((procedure? xdef?) xdef?)
 			  ((eq? xdef? #t) (lambda (n m) #t))
-			  (else def-xdef?)))
-	    )
+			  (else def-xdef?))))
 	;; Return the first (tval . lval) pair not excluded by the CPP.
 	(lambda ()
 
@@ -414,7 +421,7 @@
 	       ((apply-helper file))
 	       ((not path) (c99-err "not found: ~S" file)) ; file not found
 	       (else (set! bol #t) (push-input (open-input-file path))))
-	      (sx+attr* stmt 'path path)))
+	      (sx-attr-add stmt 'path path)))
 
 	  (define* (eval-cpp-incl/tree stmt #:optional next) ;; => stmt
 	    ;; include file as a new tree
@@ -425,10 +432,10 @@
 	      (cond
 	       ((apply-helper file) stmt)		 ; use helper
 	       ((not path) (c99-err "not found: ~S" file)) ; file not found
-	       ((with-input-from-file path run-parse) => ; add tree to stmt
-		(lambda (tree)
-		  (for-each add-define (xp1 tree))
-		  (append (sx+attr* stmt 'path path) (list tree)))))))
+	       ((with-input-from-file path run-parse) => ; add tree
+	       (lambda (tree)
+		 (for-each add-define (getdefs tree))
+		 (append (sx-attr-add stmt 'path path) (list tree)))))))
 
 	  (define (eval-cpp-stmt/code stmt) ;; => stmt
 	    (case (car stmt)
@@ -450,7 +457,7 @@
 		     (else
 		      (sferr "stmt: ~S\n" stmt)
 		      (error "1: bad cpp flow stmt")))))))
-	       
+	
 	  (define (eval-cpp-stmt/decl stmt) ;; => stmt
 	    (case (car stmt)
 	      ((if) (code-if stmt))
@@ -478,7 +485,7 @@
 		      (sferr "stmt: ~S\n" stmt)
 		      (error "2: bad cpp flow stmt")))
 		   stmt))))
-	       
+
 	  (define (eval-cpp-stmt/file stmt) ;; => stmt
 	    (case (car stmt)
 	      ((if) (cpi-push) stmt)
@@ -498,16 +505,16 @@
 	  ;; Maybe evaluate the CPP statement.
 	  (define (eval-cpp-stmt stmt)
 	    (with-throw-handler
-	     'cpp-error
-	     (lambda ()
-	       (case mode
-		 ((code) (eval-cpp-stmt/code stmt))
-		 ((decl) (eval-cpp-stmt/decl stmt))
-		 ((file) (eval-cpp-stmt/file stmt))
-		 (else (error "lang/c99 coding error"))))
-	     (lambda (key fmt . rest)
-	       (report-error fmt rest)
-	       (throw 'c99-error "CPP error"))))
+		'cpp-error
+	      (lambda ()
+		(case mode
+		  ((code) (eval-cpp-stmt/code stmt))
+		  ((decl) (eval-cpp-stmt/decl stmt))
+		  ((file) (eval-cpp-stmt/file stmt))
+		  (else (error "lang/c99 coding error"))))
+	      (lambda (key fmt . rest)
+		(report-error fmt rest)
+		(throw 'c99-error "CPP error"))))
 
 	  ;; Predicate to determine if we pass the cpp-stmt to the parser.
 	  ;; @itemize
@@ -591,7 +598,8 @@
 	       pair)
 	      ((skip-done skip-look skip)
 	       (iter (read-token)))
-	      (else (error "coding error"))))
-	  )))))
+	      (else (error "coding error")))))))
+
+    lexer))
 
 ;; --- last line ---
