@@ -28,9 +28,9 @@
   #:use-module ((srfi srfi-1) #:select (append-reverse))
   #:use-module (srfi srfi-2) ;; and-let*
   #:use-module (sxml fold)
-  #:use-module (ice-9 popen)		; gen-cc-defs
-  #:use-module (ice-9 rdelim)		; gen-cc-defs
-  #:use-module (ice-9 regex)		; gen-cc-defs
+  #:use-module (ice-9 popen)		; gen-gcc-cpp-defs
+  #:use-module (ice-9 rdelim)		; gen-gcc-cpp-defs
+  #:use-module (ice-9 regex)		; get-gcc-cpp-defs
 )
 
 ;; include-helper for C99 std
